export enum Arg {
  addConnection = '--addConnection',
  name = '--name',
  url = '--url',
  appId = '--appId',
  masterKey = '--masterKey',
  jsKey = '--jsKey',
  useConnection = '--useConnection',
  createRandCompany = '--createRandCompany',
  noContracts = '--noContracts',
  noCreditApps = '--noCreditApps',
  noEstimates = '--noEstimates',
  noMeasureItems = '--noMeasureItems',
  noOffices = '--noOffices',
  noUsers = '--noUsers',
  partnerTag = '--partnerTag',
}

// TypeGuard for Arg
const isArg = (arg: Arg | string | number): arg is Arg =>
  Object.values(Arg).includes(arg as Arg);

export const ArgValType = new Map<Arg, 'string' | 'integer' | null>([
  [Arg.addConnection, null],
  [Arg.name, 'string'],
  [Arg.url, 'string'],
  [Arg.appId, 'string'],
  [Arg.masterKey, 'string'],
  [Arg.jsKey, 'string'],
  [Arg.useConnection, null],
  [Arg.createRandCompany, null],
  [Arg.noContracts, 'integer'],
  [Arg.noCreditApps, 'integer'],
  [Arg.noEstimates, 'integer'],
  [Arg.noMeasureItems, 'integer'],
  [Arg.noOffices, 'integer'],
  [Arg.noUsers, 'integer'],
  [Arg.partnerTag, 'string'],
]);

export const getValuesFromArgs = (
  userArgs: (Arg | string | number)[]
): Map<Arg, null | string | number> => {
  const args = [...userArgs]; // clone args so we don't modify orignal

  // Create a value map from args
  const values = new Map();

  for (let i = 0; i < args.length; i++) {
    const arg = args[i];
    if (isArg(arg)) {
      const t = ArgValType.get(arg);
      switch (t) {
        case null:
          values.set(arg, null);
          break;
        case 'integer': {
          const value = Number(args[i + 1]); // element after arg is the val
          if (value < 0 || isNaN(value) || !Number.isInteger(value)) {
            throw new Error(
              `Expected a positive integer for "${arg}" but received ${
                args[i + 1]
              }`
            );
          }
          values.set(arg, value);
          i++; // skip next element since it's a value and not arg
          break;
        }
        case 'string': {
          const value = args[i + 1]; // element after arg is the val
          values.set(arg, value);
          i++; // skip next element since it's a value and not arg
          break;
        }
      }
    } else {
      throw new Error(`${arg} is not a valid argument!`);
    }
  }

  return values;
};
