import { Arg } from './args';
import { LeapGeneratorDatabase } from '@/data/database';
import { CompanySeedDocument } from '@/data/types/companySeed';
import generateRandCompany from '@/util/generateRandCompany';
import { ConsoleColor } from './types';

export const createRandomCompany = async (
  args: Map<Arg, string | null | number>,
  db: LeapGeneratorDatabase
): Promise<void> => {
  // Validate args
  const required = [
    Arg.noContracts,
    Arg.noCreditApps,
    Arg.noEstimates,
    Arg.noMeasureItems,
    Arg.noOffices,
    Arg.noUsers,
  ];
  required.forEach((arg) => {
    if (!args.has(arg)) {
      console.error(new Error(`${arg} is required for --createRandCompany!`));
    }
  });

  const [
    noContracts,
    noCreditApps,
    noEstimates,
    noMeasureItems,
    noOffices,
    noUsers,
    partnerTag,
  ]: [number, number, number, number, number, number, 'Leap' | 'Prosell'] = [
    Number(args.get(Arg.noContracts)),
    Number(args.get(Arg.noCreditApps)),
    Number(args.get(Arg.noEstimates)),
    Number(args.get(Arg.noMeasureItems)),
    Number(args.get(Arg.noOffices)),
    Number(args.get(Arg.noUsers)),
    args.get(Arg.partnerTag) === 'Prosell' ? 'Prosell' : 'Leap',
  ];

  const activeQuery = db.parseconfigs.findOne({
    selector: { active: { $eq: true } },
  });
  const activeConfig = await activeQuery.exec();
  if (activeConfig) {
    const connected = await activeConfig.connect();
    if (!connected) {
      throw new Error(`Could not connect to ${activeConfig.serverUrl}`);
    }
  } else {
    throw new Error('No active parse config!');
  }

  const companyDoc: CompanySeedDocument = await db.companyseeds.getRandom();

  const consoleLog = (
    msg: string | Error | Record<string, unknown> | [],
    color?: 'normal' | 'warning' | 'danger' | 'success'
  ) => {
    let consoleColor: string;
    const colorEnd = ConsoleColor.Content + ConsoleColor.Reset;
    switch (color) {
      case 'normal':
        consoleColor = ConsoleColor.FgWhite + colorEnd;
        break;
      case 'danger':
        consoleColor = ConsoleColor.FgRed + colorEnd;
        break;
      case 'success':
        consoleColor = ConsoleColor.FgGreen + colorEnd;
        break;
      case 'warning':
        consoleColor = ConsoleColor.FgYellow + colorEnd;
        break;
      default:
        consoleColor = ConsoleColor.Reset;
    }
    console.log(consoleColor, msg);
  };

  try {
    await generateRandCompany(
      companyDoc,
      consoleLog,
      db,
      noContracts,
      noCreditApps,
      noEstimates,
      noMeasureItems,
      noOffices,
      noUsers,
      partnerTag
    );
  } catch (e) {
    console.error(e);
  }
};
