/* eslint-disable max-lines */
import fs from 'graceful-fs';
import path from 'path';
import os from 'os';
import mkdirp from 'mkdirp';
import { Observable, Subscriber } from 'rxjs';
import { createRxDatabase, RxDatabase } from 'rxdb';

import { LeapGeneratorCollections, CollectionNames } from './types/collections';
import {
  parseConfigCollectionMethods,
  parseConfigMethods,
  parseConfigSchema,
} from './schemas/parseConfigSchema';
import {
  companySeedCollectionMethods,
  companySeedMethods,
  companySeedSchema,
} from './schemas/companySeedSchema';
import {
  officeSeedCollectionMethods,
  officeSeedMethods,
  officeSeedSchema,
} from './schemas/officeSeedSchema';
import {
  userSeedCollectionMethods,
  userSeedMethods,
  userSeedSchema,
} from './schemas/userSeedSchema';
import {
  priceGuideSeedCollectionMethods,
  priceGuideSeedMethods,
  priceGuideSeedSchema,
} from './schemas/priceGuideSeedSchema';
import {
  estimateSeedCollectionMethods,
  estimateSeedMethods,
  estimateSeedSchema,
} from './schemas/estimateSeedSchema';
import {
  contactSeedCollectionMethods,
  contactSeedMethods,
  contactSeedSchema,
} from './schemas/contactSeedSchema';
import {
  addressSeedCollectionMethods,
  addressSeedMethods,
  addressSeedSchema,
} from './schemas/addressSeedSchema';
import {
  customerSeedCollectionMethods,
  customerSeedMethods,
  customerSeedSchema,
} from './schemas/customerSchema';
import {
  creditAppSeedCollectionMethods,
  creditAppSeedMethods,
  creditAppSeedSchema,
} from './schemas/creditApplicationSchema';
import {
  contractSeedCollectionMethods,
  contractSeedMethods,
  contractSeedSchema,
} from './schemas/contractSeedSchema';
import genSeedFileSchema from './schemas/seedFileSchema';
import seedAddressData from './seedData/seedAddressData';
import seedCompanyData from './seedData/seedCompanyData';
import seedContactData from './seedData/seedContactData';
import seedContractData from './seedData/seedContractData';
import seedCreditAppData from './seedData/seedCreditApplicationData';
import seedCustomerData from './seedData/seedCustomerData';
import seedEstimateData from './seedData/seedEstimateData';
import seedOfficeData from './seedData/seedOfficeData';
import seedFiles from './seedData/seedFiles';
import seedPriceGuideData from './seedData/seedPriceGuideData';
import seedUserData from './seedData/seedUserData';

export type LeapGeneratorDatabase = RxDatabase<LeapGeneratorCollections>;

export type LeapDBObserver = {
  message: string;
  progress: number;
  db?: LeapGeneratorDatabase;
};

type LoadingAction = {
  message: string;
  action: () => Promise<void>;
};

const dbDir =
  process.env.NODE_ENV === 'production'
    ? path.resolve(os.homedir(), '.leap-data-generator', 'rxdb')
    : path.resolve(process.cwd(), 'rxdb');

// If the dir for the db doesn't exist in the user's home,
// create it
if (!fs.existsSync(dbDir)) {
  mkdirp.sync(dbDir);
}

// eslint-disable-next-line max-len
export const initializeDatabase = (
  adapter: unknown,
  seedJSONPath?: string
): Observable<LeapDBObserver> =>
  new Observable((subscriber: Subscriber<LeapDBObserver>) => {
    const createDB = async () => {
      subscriber.next({
        message: 'Opening Leap Seed Database',
        progress: 0,
      });

      // Create db collections
      const db: LeapGeneratorDatabase = await createRxDatabase({
        name: `${dbDir}/leapgendb`,
        adapter,
      });

      // actions to load and seed the db
      const steps: LoadingAction[] = [
        {
          action: () =>
            db.collection({
              name: CollectionNames.ParseConfigs,
              schema: parseConfigSchema,
              methods: parseConfigMethods,
              statics: parseConfigCollectionMethods,
            }),
          message: 'Creating Parse Configurations Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.CompanySeeds,
              schema: companySeedSchema,
              methods: companySeedMethods,
              statics: companySeedCollectionMethods,
            }),
          message: 'Creating Copmany Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.OfficeSeeds,
              schema: officeSeedSchema,
              methods: officeSeedMethods,
              statics: officeSeedCollectionMethods,
            }),
          message: 'Creating Office Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.UserSeeds,
              schema: userSeedSchema,
              methods: userSeedMethods,
              statics: userSeedCollectionMethods,
            }),
          message: 'Creating User Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.PriceGuideSeeds,
              schema: priceGuideSeedSchema,
              methods: priceGuideSeedMethods,
              statics: priceGuideSeedCollectionMethods,
            }),
          message: 'Creating Price Guide Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.EstimateSeeds,
              schema: estimateSeedSchema,
              methods: estimateSeedMethods,
              statics: estimateSeedCollectionMethods,
            }),
          message: 'Creating Estimate Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.ContactSeeds,
              schema: contactSeedSchema,
              methods: contactSeedMethods,
              statics: contactSeedCollectionMethods,
            }),
          message: 'Creating Contact Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.AddressSeeds,
              schema: addressSeedSchema,
              methods: addressSeedMethods,
              statics: addressSeedCollectionMethods,
            }),
          message: 'Creating Address Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.CustomerSeeds,
              schema: customerSeedSchema,
              methods: customerSeedMethods,
              statics: customerSeedCollectionMethods,
            }),
          message: 'Creating Customer Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.CreditAppSeeds,
              schema: creditAppSeedSchema,
              methods: creditAppSeedMethods,
              statics: creditAppSeedCollectionMethods,
            }),
          message: 'Creating Credit Apllication Seed Data Collection',
        },
        {
          action: () =>
            db.collection({
              name: CollectionNames.ContractSeeds,
              schema: contractSeedSchema,
              methods: contractSeedMethods,
              statics: contractSeedCollectionMethods,
            }),
          message: 'Creating Contract Seed Data Collection',
        },
        {
          action: () => {
            const {
              seedFileCollectionMethods,
              seedFileMethods,
              seedFileSchema,
            } = genSeedFileSchema(seedJSONPath);
            return db.collection({
              name: CollectionNames.SeedFiles,
              schema: seedFileSchema,
              methods: seedFileMethods,
              statics: seedFileCollectionMethods,
            });
          },
          message: 'Creating Seed File Collection',
        },
        // Setup default seed files
        {
          action: () => seedFiles(db),
          message: 'Setting Up Default Seed Files',
        },
        // Seed collections from json files
        {
          action: () => seedCompanyData(db),
          message: 'Populating Company Seed Data',
        },
        {
          action: () => seedUserData(db),
          message: 'Populating User Seed Data',
        },
        {
          action: () => seedOfficeData(db),
          message: 'Populating Office Seed Data',
        },
        {
          action: () => seedPriceGuideData(db),
          message: 'Populating Price Guide Seed Data',
        },
        {
          action: () => seedEstimateData(db),
          message: 'Populating Estimate Seed Data',
        },
        {
          action: () => seedContactData(db),
          message: 'Populating Contact Seed Data',
        },
        {
          action: () => seedAddressData(db),
          message: 'Populating Address Seed Data',
        },
        {
          action: () => seedCustomerData(db),
          message: 'Populating Customer Seed Data',
        },
        {
          action: () => seedCreditAppData(db),
          message: 'Populating Credit App Seed Data',
        },
        {
          action: () => seedContractData(db),
          message: 'Populating Contract Seed Data',
        },
        // If there's an active conneciton when the db loads, try connecting to
        // Parse and update status
        {
          action: async () => {
            const activeQuery = db.parseconfigs.findOne({
              selector: { active: { $eq: true } },
            });
            const activeConfig = await activeQuery.exec();
            if (activeConfig) {
              activeConfig.connect();
            }
          },
          message: 'Loading Previously Active Connection',
        },
      ];

      // Itterate through and run loading actions. Dispatch status back to
      // db$ at the beginning of each run
      for (let i = 0; i < steps.length; i++) {
        const { action, message } = steps[i];
        const progress = Math.floor((i / steps.length) * 100);
        subscriber.next({
          message,
          progress,
        });
        try {
          await action();
        } catch (e) {
          console.error(e, message);
          subscriber.next({ message: e.mesage, progress });
        }
      }

      subscriber.next({
        message: 'Leap Seed Database Scaffolded',
        progress: 100,
        db,
      });

      subscriber.complete();
    };
    createDB();
  });
