import { RxJsonSchema } from 'rxdb';
import Parse from 'parse/node';
import Company from '../../parse/models/company';
import {
  CompanySeedCollection,
  CompanySeedCollectionMethods,
  CompanySeedDocType,
  CompanySeedDocument,
  CompanySeedMethods,
} from '../types/companySeed';

export const companySeedSchema: RxJsonSchema<CompanySeedDocType> = {
  title: 'Parse Config Schema',
  description: 'Connection information for the Parse/LeapOne Server',
  version: 0,
  keyCompression: true,
  type: 'object',
  properties: {
    id: {
      type: 'string',
      primary: true,
    },
    companyName: { type: 'string' },
    maxActiveUsers: { type: 'number' },
    officeName: { type: 'string' },
    username: { type: 'string' },
    password: { type: 'string' },
    nameFirst: { type: 'string' },
    nameLast: { type: 'string' },
    email: { type: 'string' },
  },
  required: [],
};

export const companySeedMethods: CompanySeedMethods = {
  async checkExists(this: CompanySeedDocument): Promise<boolean> {
    const name = this.companyName;
    const query = new Parse.Query(Company);
    query.equalTo('name', name);
    const result = await query.first();
    return !!result;
  },
  async createCompany(
    this: CompanySeedDocument,
    maxActiveUsers: number,
    partnerTag: 'Leap' | 'Prosell'
  ): Promise<Company> {
    const {
      companyName,
      officeName,
      username,
      password,
      nameFirst,
      nameLast,
      email,
    } = this;
    return Company.create({
      companyName,
      officeName,
      username,
      password,
      nameFirst,
      nameLast,
      email,
      maxActiveUsers,
      partnerTag,
    });
  },
  getAccountCredentials(
    this: CompanySeedDocument
  ): { username: string; password: string } {
    const { username, password } = this;
    return { username, password };
  },
};

export const companySeedCollectionMethods: CompanySeedCollectionMethods = {
  async countAllDocuments(this: CompanySeedCollection) {
    const allDocs = await this.find().exec();
    return allDocs.length;
  },
  async getExisting(this: CompanySeedCollection) {
    const parseQuery = new Parse.Query(Company);
    parseQuery.limit(10000);
    const results = await parseQuery.find({ useMasterKey: true });

    const nameStatement = results.map((company) => ({
      companyName: { $eq: company.get('name') },
    }));

    const query = this.find({
      selector: { $or: nameStatement },
    });

    return query.exec();
  },
  async getRandom(this: CompanySeedCollection) {
    const parseQuery = new Parse.Query(Company);
    parseQuery.limit(10000);
    const results = await parseQuery.find({ useMasterKey: true });
    const nameStatement = results.map((company) => ({
      companyName: { $ne: company.get('name') },
    }));

    const uniqueCompanies = this.find({
      selector: {
        $and: nameStatement,
      },
    });
    const count = (await uniqueCompanies.exec()).length;
    if (count === 0)
      throw new Error('All seed companies have already been created!');

    const randId = Math.floor(Math.random() * count + 1);
    const query = this.findOne({
      selector: {
        $and: nameStatement,
      },
      skip: randId - 1,
    });

    const companySeed = await query.exec();

    if (companySeed) {
      return companySeed;
    } else {
      throw new Error('Could not get Company Seed');
    }
  },
};
