import path from 'path';
import { RxJsonSchema } from 'rxdb';
import Estimate, { IMockEstimate } from '../../parse/models/estimate';
import Office from '../../parse/models/office';
import Company from '../../parse/models/company';
import {
  EstimateSeedCollection,
  EstimateSeedCollectionMethods,
  EstimateSeedDocType,
  EstimateSeedDocument,
  EstimateSeedMethods,
} from '../types/estimateSeed';
import makeParseFile from '@/util/makeParseFile';
import { genRandomId } from '@/util/getRandom';

export const estimateSeedSchema: RxJsonSchema<EstimateSeedDocType> = {
  title: 'Estimate Seed Schema',
  description: 'Seed data for generating estimates',
  version: 0,
  keyCompression: true,
  type: 'object',
  properties: {
    id: {
      type: 'string',
      primary: true,
    },
    activeProductCategories: {
      type: 'array',
      items: {
        type: 'string',
      },
    },
    addedMeasureSheetItems: {
      type: 'array',
      items: {
        type: 'object',
        properties: {
          category: { type: 'string' },
          quantity: { type: 'number' },
          additionalValues: { type: 'object' },
          itemObjectId: { type: 'string' },
          activeObjecttId: { type: 'string' },
          isActive: { type: 'boolean' },
        },
        additionalProperties: true,
      },
    },
    contractPDF: {
      type: 'string',
    },
    creditAppPDF: {
      type: 'string',
    },
    downPayment: {
      type: 'number',
    },
    financeAmount: {
      type: 'number',
    },
    isSale: {
      type: 'boolean',
    },
    leadResults: {
      type: 'array',
      items: {
        type: 'object',
        properties: {
          values: { type: 'array', items: { type: 'string' } },
          apiValue: { type: 'string' },
          id: { type: 'number' },
          order: { type: 'number' },
          title: { type: 'string' },
          shouldFetch: { type: 'boolean' },
          inputType: { type: 'string' },
          value: { type: 'string' },
          endpoint: { type: 'string' },
          required: { type: 'boolean' },
        },
      },
    },
    resultFullString: { type: 'string' },
    resultPDF: { type: 'string' },
    saleAmount: { type: 'number' },
    selectedEmailLinks: { type: 'object' },
  },
};

const testDocPath: string = path.join(
  __dirname,
  '../../../assets/test-doc.pdf'
);
export const estimateSeedMethods: EstimateSeedMethods = {
  async createEstimate(
    this: EstimateSeedDocument,
    office: Office,
    company: Company,
    contractPDFPath = testDocPath,
    contractVerifyPDFPath = testDocPath,
    creditAppPDFPath = testDocPath,
    proposalPDFPath = testDocPath,
    proposalVerifyPDFPath = testDocPath,
    reportPDFPath = testDocPath,
    resultPDFPath = testDocPath
  ): Promise<Estimate> {
    const docPrefix = genRandomId();
    const {
      activeProductCategories,
      addedMeasureSheetItems,
      downPayment,
      financeAmount,
      isSale,
      leadResults,
      resultFullString,
      saleAmount,
      selectedEmailLinks,
    } = this;
    const newEstimate: IMockEstimate = {
      activeProductCategories,
      addedMeasureSheetItems,
      availability: {},
      company,
      contractPDF: await makeParseFile(
        contractPDFPath,
        `${docPrefix}-contractPDF`
      ),
      contractVerifyPDF: await makeParseFile(
        contractVerifyPDFPath,
        `${docPrefix}-contractVerifyPDF`
      ),
      creditAppPDF: await makeParseFile(
        creditAppPDFPath,
        `${docPrefix}-creditAppPDF`
      ),
      demoResult: 0,
      // docuSign: {},
      // docusignCompleted: {},
      // docusignEnvelopeIds: [],
      // docusignObjects: {},
      // docusignResponse: {},
      downPayment,
      financeAmount,
      isSale,
      leadResults,
      office,
      paymentRequest: {},
      proposalPDF: await makeParseFile(
        proposalPDFPath,
        `${docPrefix}-proposalPDF`
      ),
      proposalVerifyPDF: await makeParseFile(
        proposalVerifyPDFPath,
        `${docPrefix}-proposalVerifyPDF`
      ),
      reportPDF: await makeParseFile(reportPDFPath, `${docPrefix}-reportPDF`),
      resultFullString,
      resultLastPriceLeft: '',
      resultLeadResult: '',
      resultNote: '',
      resultPDF: await makeParseFile(resultPDFPath, `${docPrefix}-resultPDF`),
      saleAmount,
      selectedEmailLinks,
    };
    return Estimate.create(newEstimate);
  },
};

export const estimateSeedCollectionMethods: EstimateSeedCollectionMethods = {
  async countAllDocuments(this: EstimateSeedCollection) {
    const allDocs = await this.find().exec();
    return allDocs.length;
  },
  async getRandom(this: EstimateSeedCollection) {
    const estimates = this.find();
    const count = (await estimates.exec()).length;
    const randId = Math.floor(Math.random() * count + 1);
    const query = this.findOne({
      selector: { id: { $gt: 0 } },
      skip: randId - 1,
    });

    const estimateSeed = await query.exec();

    if (estimateSeed) {
      return estimateSeed;
    } else {
      throw new Error('Could not get Estimate Seed');
    }
  },
};
