import fetch from 'node-fetch';
import { RxJsonSchema } from 'rxdb';
import {
  ParseConfigDocType,
  ParseConfigMethods,
  ParseConfigCollectionMethods,
  ParseConfigCollection,
  ParseConfigDocument,
} from '../types/parseConfig';
import InitializeParse from '../../parse';

export const parseConfigSchema: RxJsonSchema<ParseConfigDocType> = {
  title: 'Parse Config Schema',
  description: 'Connection information for the Parse/LeapOne Server',
  version: 0,
  keyCompression: true,
  type: 'object',
  properties: {
    id: {
      type: 'string',
      primary: true,
    },
    serverName: {
      type: 'string',
    },
    appId: {
      type: 'string',
    },
    jsKey: {
      type: 'string',
    },
    masterKey: {
      type: 'string',
    },
    serverUrl: {
      type: 'string',
    },
    active: {
      type: 'boolean',
    },
  },
  indexes: ['serverName'],
  required: ['appId', 'masterKey', 'serverUrl'],
};

export const parseConfigMethods: ParseConfigMethods = {
  // Class method to connect to parse server described in config
  async connect(this: ParseConfigDocument): Promise<boolean> {
    if (await this.checkOnline()) {
      const oldConnection = await this.collection
        .findOne({
          selector: {
            active: true,
            id: { $ne: this.id },
          },
        })
        .exec();
      if (oldConnection) {
        await oldConnection.update({
          $set: {
            active: false,
          },
        });
      }
      try {
        const { appId, jsKey, masterKey, serverUrl } = this;
        InitializeParse({
          appId,
          jsKey,
          masterKey,
          serverUrl,
        });
        await this.update({
          $set: { active: true },
        });
        return true;
      } catch (e) {
        console.log(e);
        return false;
      }
    }
    return false;
  },
  // Class method pings the server and returns true if it responds
  async checkOnline(this: ParseConfigDocument) {
    const { serverUrl } = this;
    try {
      await fetch(`${serverUrl}/health`);
      return true;
    } catch (e) {
      console.log(e);
      return false;
    }
  },
};

export const parseConfigCollectionMethods: ParseConfigCollectionMethods = {
  // tslint:disable-next-line: object-literal-shorthand
  async countAllDocuments(this: ParseConfigCollection) {
    const allDocs = await this.find().exec();
    return allDocs.length;
  },
};
