import { RxJsonSchema, JsonSchema } from 'rxdb';
import Parse from 'parse/node';
import Company from '../../parse/models/company';
import SSMeasureSheetItem from '../../parse/models/measureSheetItem';
import Office from '../../parse/models/office';
import {
  PriceGuideSeedCollectionMethods,
  PriceGuideSeedCollection,
  PriceGuideSeedDocType,
  PriceGuideSeedDocument,
  PriceGuideSeedMethods,
} from '../types/priceGuideSeed';

const PriceItemSchema: JsonSchema = {
  type: 'object',
  properties: {
    accessories: {
      type: 'array',
      items: { type: 'object', additionalProperties: true },
    },
    accessoriesPricesDict: {
      type: 'object',
      additionalProperties: true,
    },
    accessoryPrices: {
      type: 'array',
      items: { type: 'object', additionalProperties: true },
    },
    additionalDetails: {
      type: 'array',
      items: {
        type: 'object',
        properties: {
          cellType: { type: 'string' },
          defaultValue: { type: 'string' },
          inputType: { type: 'string' },
          note: { type: 'string' },
          objectId: { type: 'string' },
          placeholder: { type: 'string' },
          required: { type: 'boolean' },
          shouldCopy: { type: 'boolean' },
          title: { type: 'string' },
        },
      },
    },
    category: { type: 'string' },
    commissionable: { type: 'boolean' },
    disabledParents: { type: 'array', items: { type: 'string' } },
    displayTitle: { type: 'string' },
    identifier: { type: 'string' },
    info: { type: 'string' },
    isAccessory: { type: 'boolean' },
    itemCode: { type: 'string' },
    itemCodes: { type: 'object', additionalProperties: true },
    itemPrices: {
      type: 'array',
      items: {
        type: 'object',
        properties: {
          total: { type: 'number' },
        },
      },
    },
    measurementType: { type: 'string' },
    name: { type: 'string' },
    orderNumber: { type: 'number' },
    parArray: { type: 'array', items: { type: 'number' } },
    parentItem: { type: 'string' },
    percentagePrice: { type: 'boolean' },
    placeholders: {
      type: 'array',
      items: {
        type: 'object',
        properties: {
          placeholder: { type: 'string' },
          replacement: { type: 'string' },
          id: { type: 'string' },
        },
      },
    },
    pricesDict: { type: 'object', additionalProperties: true },
    shouldShowSwitch: { type: 'number' },
    subCategory: { type: 'string' },
    subCategory2: { type: 'string' },
  },
};

export const priceGuideSeedSchema: RxJsonSchema<PriceGuideSeedDocType> = {
  title: 'Measure Sheet Seed Item Schema',
  description: 'Measuresheet item with their child price guide items',
  version: 0,
  keyCompression: true,
  type: 'object',
  properties: {
    id: {
      type: 'string',
      primary: true,
    },
    itemCount: { type: 'integer' },
    items: {
      type: 'array',
      items: PriceItemSchema,
    },
    category: { type: 'string' },
    subCategory: { type: 'string' },
    subSubCategory: { type: 'string' },
    subSubcategories: { type: 'string' },
    itemName: { type: 'string' },
    itemNote: { type: 'string' },
    measurementType: { type: 'string' },
    shouldShowSwitch: { type: 'boolean' },
    orderNumber: { type: 'number' },
  },
  required: [],
};

export const priceGuideSeedMethods: PriceGuideSeedMethods = {
  async checkExists(this: PriceGuideSeedDocument): Promise<boolean> {
    const name = this.itemName;
    const query = new Parse.Query(SSMeasureSheetItem);
    query.equalTo('itemName', name);
    const result = await query.first();
    return !!result;
  },
  async createPriceGuideItem(
    this: PriceGuideSeedDocument,
    company: Company,
    offices: Office[],
    token: string
  ): Promise<SSMeasureSheetItem> {
    const {
      category,
      itemName,
      itemNote,
      items,
      measurementType,
      orderNumber,
      shouldShowSwitch,
      subCategory,
      subSubcategories,
      subSubCategory,
    } = this;
    return SSMeasureSheetItem.create(
      {
        company,
        category,
        includedOffices: offices,
        itemName,
        itemNote,
        items,
        measurementType,
        orderNumber,
        shouldShowSwitch,
        subCategory,
        subSubcategories,
        subSubCategory,
      },
      company,
      offices,
      token
    );
  },
};

// eslint-disable-next-line max-len
export const priceGuideSeedCollectionMethods: PriceGuideSeedCollectionMethods = {
  async countAllDocuments(this: PriceGuideSeedCollection) {
    const allDocs = await this.find().exec();
    return allDocs.length;
  },
  async getRandom(this: PriceGuideSeedCollection, company: Company) {
    const parseQuery = new Parse.Query(SSMeasureSheetItem);
    parseQuery.equalTo('company', company).limit(10000);
    const results = await parseQuery.find({ useMasterKey: true });

    const nameStatement = results.map((measureSheetItem) => ({
      itemName: { $ne: measureSheetItem.get('itemName') },
    }));

    const uniqueNamed = this.find({
      selector: { $and: nameStatement },
    });
    const count = (await uniqueNamed.exec()).length;
    if (count === 0)
      throw new Error(
        // eslint-disable-next-line max-len
        'All seed measure sheet items have already been created in this company!'
      );

    const randId = Math.floor(Math.random() * count + 1);
    const query = this.findOne({
      selector: { $and: nameStatement },
      skip: randId - 1,
    });

    const priceItem = await query.exec();

    if (priceItem) {
      return priceItem;
    } else {
      throw new Error('Could not get Price Guide Seed Item');
    }
  },
};
