import fs from 'graceful-fs';
import path from 'path';
import { RxJsonSchema } from 'rxdb';
import {
  SeedFileCollection,
  SeedFileCollectionMethods,
  SeedFileDocType,
  SeedFileDocument,
  SeedFileMethods,
} from '../types/seedFile';

export default (
  seedJSONPath = path.join(__dirname, '../../../seedJSON/')
): {
  seedFileSchema: RxJsonSchema<SeedFileDocType>;
  seedFileCollectionMethods: SeedFileCollectionMethods;
  seedFileMethods: SeedFileMethods;
} => {
  const seedFileSchema: RxJsonSchema<SeedFileDocType> = {
    title: 'Seed File Schema',
    description:
      'The filename & last modified for the seed JSON data for each collection',
    version: 0,
    keyCompression: true,
    type: 'object',
    properties: {
      collectionName: { type: 'string', primary: true },
      fileLastModified: { type: 'string' },
      fileName: { type: 'string' },
    },
  };

  const getFileModifiedTime = (filePath: string): Promise<Date> =>
    new Promise((resolve, reject) => {
      fs.stat(filePath, (err, stats) => {
        if (err) {
          console.error(err);
          reject(err);
        }
        const { mtime } = stats;
        const modifiedTime = new Date(mtime);
        resolve(modifiedTime);
      });
    });

  const getFilePath = (fileName: string) =>
    path.join(seedJSONPath, `${fileName}`);

  const readJSONFile = <T>(location: string): Promise<T> =>
    new Promise((resolve, reject) => {
      console.log(`Reading file ${location}`);
      try {
        fs.readFile(location, 'utf8', (error, data) => {
          if (error) throw error;
          try {
            console.log(`File read, parsing JSON`);
            const parsedData = JSON.parse(data);
            console.log('JSON parsed!');
            resolve(parsedData as T);
          } catch (e) {
            console.error(`${e.name}: ${e.message}`);
            reject(e);
            throw e;
          }
        });
      } catch (err) {
        reject(err);
        console.error(err);
        reject('An unkown error occured');
        throw err;
      }
    });

  const seedFileMethods: SeedFileMethods = {
    async readFile<T>(this: SeedFileDocument) {
      const filePath = getFilePath(this.fileName);
      return readJSONFile<T>(filePath);
    },
    async updateLastModified(this: SeedFileDocument) {
      const lastModified = await getFileModifiedTime(
        getFilePath(this.fileName)
      );
      return this.update({
        $set: { fileLastModified: lastModified.toISOString() },
      });
    },
    async wasFileModified(this: SeedFileDocument) {
      // If we've never read the file before, return true
      if (this.fileLastModified === '') {
        return true;
      }
      const prevModified = new Date(this.fileLastModified);
      const lastModified = await getFileModifiedTime(
        getFilePath(this.fileName)
      );
      return lastModified > prevModified;
    },
  };

  const seedFileCollectionMethods: SeedFileCollectionMethods = {
    async countAllDocuments(this: SeedFileCollection) {
      const allDocs = await this.find().exec();
      return allDocs.length;
    },
  };

  return { seedFileSchema, seedFileCollectionMethods, seedFileMethods };
};
