import { RxJsonSchema } from 'rxdb';
import Parse from 'parse/node';
import Office from '../../parse/models/office';
import User from '../../parse/models/user';
import {
  UserSeedCollection,
  UserSeedCollectionMethods,
  UserSeedDocType,
  UserSeedDocument,
  UserSeedMethods,
} from '../types/userSeed';

export const userSeedSchema: RxJsonSchema<UserSeedDocType> = {
  title: 'Parse Config Schema',
  description: 'Connection information for the Parse/LeapOne Server',
  version: 0,
  keyCompression: true,
  type: 'object',
  properties: {
    id: {
      type: 'string',
      primary: true,
    },
    userName: { type: 'string' },
    email: { type: 'string' },
    nameFirst: { type: 'string' },
    nameLast: { type: 'string' },
    phoneNumber: { type: 'string' },
    licenseNumber: { type: 'number' },
    password: { type: 'string' },
  },
  required: [],
};

export const userSeedMethods: UserSeedMethods = {
  async checkExists(this: UserSeedDocument): Promise<boolean> {
    const name = this.userName;
    const query = new Parse.Query(User);
    query.equalTo('username', name);
    const result = await query.first();
    return !!result;
  },
  async createUser(
    this: UserSeedDocument,
    office: Office,
    token: string
  ): Promise<User> {
    const {
      email,
      licenseNumber,
      nameFirst,
      nameLast,
      password,
      phoneNumber,
      userName: username,
    } = this;
    return User.create(
      {
        allowedOffices: [office.id],
        apiCredentials: [],
        email,
        licenseNumber,
        nameFirst,
        nameLast,
        office: office.id,
        password,
        phoneNumber,
        username,
      },
      token
    );
  },
  getAccountCredentials(
    this: UserSeedDocument
  ): { username: string; password: string } {
    const { userName: username, password } = this;
    return { username, password };
  },
};

export const userSeedCollectionMethods: UserSeedCollectionMethods = {
  async countAllDocuments(this: UserSeedCollection) {
    const allDocs = await this.find().exec();
    return allDocs.length;
  },
  async getExisting(this: UserSeedCollection) {
    const parseQuery = new Parse.Query(User);
    parseQuery.limit(10000);
    const results = await parseQuery.find();

    const nameStatement = results.map((user) => ({
      userName: { $eq: user.get('username') },
    }));

    const query = this.find({
      selector: { $or: nameStatement },
    });

    return query.exec();
  },
  async getRandom(this: UserSeedCollection) {
    const parseQuery = new Parse.Query(User);
    parseQuery.limit(10000);
    const results = await parseQuery.find({ useMasterKey: true });

    const nameStatement = results.map((user) => ({
      userName: { $ne: user.get('username') },
    }));

    const uniqueQuery = this.find({
      selector: { $and: nameStatement },
    });
    const count = (await uniqueQuery.exec()).length;
    if (count === 0)
      throw new Error('All seed users have already been created!');

    const randId = Math.floor(Math.random() * count + 1);
    const query = this.findOne({
      selector: { $and: nameStatement },
      skip: randId - 1,
    });

    const userSeed = await query.exec();

    if (userSeed) {
      return userSeed;
    } else {
      throw new Error('Error getting user seed!');
    }
  },
};
