import { LeapGeneratorDatabase } from '../database';
import { CollectionNames } from '../types/collections';
import { RxDocument } from 'rxdb';

export const seedCollection = async <SeedDataType>(
  db: LeapGeneratorDatabase,
  collection: CollectionNames,
  typeguard: (seedData: SeedDataType[]) => boolean
): Promise<void> => {
  const docCount = await db[collection]?.countAllDocuments();
  const seedFileDoc = await db.seedfiles.findOne(collection).exec();
  if (seedFileDoc) {
    const modified = await seedFileDoc.wasFileModified();

    // If the collection is empty, or the file has changed since last processed,
    // seed the database
    if (docCount === 0 || modified) {
      // Process JSON file into JS
      let seedData: SeedDataType[];
      try {
        seedData = await seedFileDoc.readFile<SeedDataType[]>();
      } catch (e) {
        throw new Error(`Failed to parse ${collection} json. ${e.message}`);
      }
      // Make sure the data is type safe
      if (typeguard(seedData)) {
        // If there's exisitng data, clear it before re-seeding
        if (docCount !== 0) {
          const existingDocs = await db[collection].find().exec();
          if (Array.isArray(existingDocs)) {
            await Promise.all(
              ((existingDocs as unknown) as RxDocument[]).map(
                (doc: RxDocument) => doc.remove()
              )
            );
          }
        }

        // Seed DB
        const seedDocs = seedData.map((seed, index) => ({
          id: index.toString(),
          ...seed,
        }));
        try {
          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
          // @ts-ignore
          const results = await db[collection].bulkInsert(seedDocs);
          console.log(
            // eslint-disable-next-line max-len
            `Import results for ${collection}, ${seedDocs.length} items - success: ${results.success.length} error: ${results.error.length}`
          );
        } catch (e) {
          console.error(e);
        }

        // Record the time data was parsed to check against file rev in future
        await seedFileDoc.updateLastModified();
      }
    }
  }
};
