import Parse from 'parse/node';
import { v4 as uuid } from 'uuid';
import Company from './company';
import SSAddress from './address';
import SSContact from './contact';
import SSEstimate from './estimate';
import User from './user';
import { IMockCustomer } from '../../data/types/customerSeed';

export interface ICustomerAttributes {
  apiSourceData: {
    apiSource: string;
    data: {
      appointment: {
        address: string;
        addressLine2: string;
        appointmentDateTime: Date;
        city: string;
        contactId: string; // GUID
        dateIssued: string;
        directions: string;
        hideInScheduler: boolean;
        id: string; // GUID
        inquiryId: string; // GUID
        lastModifiedDate: Date;
        notes: string;
        result: string;
        resultReason: string;
        salesperson: string;
        salesperson2: string;
        setBy: string;
        state: string;
        type: string;
        zip: string;
      };
      contacts: {
        address: string;
        addressLine2: string;
        city: string;
        companyId: string;
        contactBusinessName: string;
        contactType: string;
        email1: string;
        email2: string;
        email3: string;
        firstName: string;
        id: string; // GUID
        lastModifiedDateTime: Date;
        lastName: string;
        phoneNumbers: {
          contactId: string;
          freeFormNumber: string;
          id: string;
          isPrimary: boolean;
          phoneNumberDigits: string;
          type: string;
        }[];
        state: string;
        zip: string;
      }[];
      inquiry: {
        canvasser: string;
        contactId: string; // GUID
        division: string;
        id: string; // GUID
        inquiryDateTime: Date;
        lastModifiedDateTime: Date;
        leadPaintYearHomeBuilt: string;
        notes: string;
        primaryLeadSource: string;
        productInterests: string;
        promoter: string;
        secondaryLeadSource: string;
        takenBy: string;
        telemarketer: string;
      };
    };
  };
  addresses: SSAddress[];
  contacts: SSContact[];
  marketSharpAppointmentId: string; // GUID
  nameLast: string;
  estimate: SSEstimate;
  marketsharpId: string; // GUID
  identifier: string;
  isSpouse: boolean;
  createdBy: User;
  demoResult: 0 | 1;
  nameFirst: string;
  nameFirstLowerCase: string;
  nameLastLowerCase: string;
  company: Company;
}

const getRandDate = () => new Date(Date.now());
const getRandNum = (max: number, padding: number): string => {
  const randNum = Math.floor(Math.random() * max + 1).toString();
  const paddedNum = randNum.padStart(padding, '0');
  return paddedNum;
};

class SSCustomer extends Parse.Object {
  constructor(options: Record<string, unknown>) {
    super('SSCustomer', options);
  }

  attributes!: ICustomerAttributes;

  static async create(
    customer: IMockCustomer,
    contactPhone: SSContact,
    contactEmail: SSContact,
    address: SSAddress,
    estimate: SSEstimate,
    company: Company,
    creator: User,
    token: string
  ): Promise<SSCustomer> {
    const { firstName, lastName } = customer;
    const [phone, email, state, city, zip, street] = [
      contactPhone.get('contact'),
      contactEmail.get('contact'),
      address.get('theState'),
      address.get('city'),
      address.get('zipCode'),
      address.get('street'),
    ];

    const options = { sessionToken: token };

    const mockCustomer = {
      apiSourceData: {
        apiSource: 'Marketsharp',
        data: {
          appointment: {
            address: street,
            addressLine2: '',
            appointmentDateTime: getRandDate(),
            city,
            contactId: uuid(), // GUID
            dateIssued: getRandDate(),
            directions:
              // eslint-disable-next-line max-len
              'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.',
            hideInScheduler: false,
            id: uuid(), // GUID
            inquiryId: uuid(), // GUID
            lastModifiedDate: uuid(),
            notes:
              // eslint-disable-next-line max-len
              'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.',
            result: getRandNum(10, 1),
            resultReason: '',
            salesperson: `${creator.get('nameFirst')} ${creator.get(
              'nameLast'
            )}`,
            salesperson2: '',
            setBy: `${firstName} ${lastName}`,
            state,
            type: 'Sales Call',
            zip,
          },
          contacts: [
            {
              address: street,
              addressLine2: '',
              city,
              companyId: getRandNum(9999, 4),
              contactBusinessName: '',
              contactType: 'CUSTOMER',
              email1: email,
              email2: '',
              email3: '',
              firstName,
              id: uuid(), // GUID
              lastModifiedDateTime: getRandDate(),
              lastName,
              phoneNumbers: [
                {
                  contactId: uuid(),
                  freeFormNumber: phone,
                  id: uuid(),
                  isPrimary: true,
                  phoneNumberDigits: Number(phone),
                  type: 'Cell Phone',
                },
              ],
              state,
              zip,
            },
          ],
          inquiry: {
            canvasser: '',
            contactId: uuid(), // GUID
            division: '',
            id: uuid(), // GUID
            inquiryDateTime: getRandDate(),
            lastModifiedDateTime: getRandDate(),
            leadPaintYearHomeBuilt: '',
            notes:
              // eslint-disable-next-line max-len
              'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.',
            primaryLeadSource: 'Direct Mail',
            productInterests: 'Home Improvement',
            promoter: '',
            secondaryLeadSource: '',
            takenBy: getRandNum(20, 0),
            telemarketer: '',
          },
        },
      },
      addresses: [address],
      contacts: [contactEmail, contactPhone],
      marketsharpAppointmentId: uuid(), // GUID
      nameLast: lastName,
      estimate,
      marketsharpId: uuid(), // GUID
      identifier: uuid(),
      isSpouse: false,
      createdBy: creator,
      demoResult: 0,
      nameFirst: firstName,
      nameFirstLowerCase: firstName.toLowerCase(),
      nameLastLowerCase: lastName.toLowerCase(),
      company,
    };
    const newCustomer = new SSCustomer(mockCustomer);

    await newCustomer.save(null, options);

    return newCustomer;
  }
}

export default SSCustomer;
