/* eslint-disable max-lines */
/* eslint-disable no-underscore-dangle */
import fs from 'graceful-fs';
import moment from 'moment';
import path from 'path';
import Parse from 'parse/node';
import getSessionToken from '@/parse/helpers/getSessionToken';
import User from '@/parse/models/user';
import { LeapGeneratorDatabase } from '@/data/database';
import Office from '@/parse/models/office';
import Company from '@/parse/models/company';
import PartnerCreditApp from '@/parse/models/partnerCreditApplication';
import Customer from '@/parse/models/customer';
import CreditApp from '@/parse/models/creditApplication';
import { CompanySeedDocument } from '@/data/types/companySeed';
import MeasureSheetItem from '@/parse/models/measureSheetItem';
import { getRand } from './getRandom';
import ContractObject, {
  IMockContractObject,
} from '@/parse/models/contractObject';
import { getDuration, getNow } from '@/util/consoleTimer';
import { UserSeedDocument } from '@/data/types/userSeed';
import makeParseFile from '@/util/makeParseFile';

const generateRandCompany = async (
  companySeed: CompanySeedDocument,
  consoleLog: (
    // eslint-disable-next-line @typescript-eslint/ban-types
    msg: string | Error | {} | [],
    color?: 'normal' | 'warning' | 'danger' | 'success'
  ) => void,
  db: LeapGeneratorDatabase,
  noContracts: number,
  noCreditApps: number,
  noEstimates: number,
  noMeasureItems: number,
  noOffices: number,
  noUsers: number,
  partnerTag: 'Leap' | 'Prosell',
  creditLogoPath = path.join(__dirname, '../../assets/test-credit-logo.png'),
  testPDFPath = path.join(__dirname, '../../assets/test-doc.pdf')
): Promise<void> => {
  try {
    // Get our test assets from disk and create Parse file objects
    const creditLogoFile = await makeParseFile(
      creditLogoPath,
      'test-credit-logo.png'
    );
    const testPdfFile = await makeParseFile(testPDFPath, 'test-doc.pdf');

    const testPdfThumbnailUrl = path.resolve(
      __dirname,
      '../../assets/test-doc.png'
    );
    const testPdfThumbnail = fs.readFileSync(testPdfThumbnailUrl, {
      encoding: 'base64',
    });
    const testPdfThumbnailFile = new Parse.File('test-doc.png', {
      base64: testPdfThumbnail,
    });

    consoleLog('Creating company');
    const createCompanyStart = getNow();
    const {
      companyName,
      officeName,
      username,
      password,
      nameFirst,
      nameLast,
      email,
    } = companySeed;
    const company = await Company.create({
      companyName,
      officeName,
      username,
      password,
      nameFirst,
      nameLast,
      email,
      maxActiveUsers: Number(noUsers),
      partnerTag,
    });
    const initialOffice = company.get('offices')[0];
    const iUserQuery = new Parse.Query(User);
    iUserQuery.equalTo('company', company);
    const initialUser = await iUserQuery.first({ useMasterKey: true });
    consoleLog(
      `Company: ${company.get(
        'name'
      )}, created with Office: ${initialOffice.get(
        'name'
      )}, and User: ${initialUser?.get('username')}`,
      'success'
    );
    getDuration(createCompanyStart, 'Create Company');
    const sessionToken = await getSessionToken(username, password);
    const offices: Office[] = [initialOffice];
    const users = [initialUser];
    const userSeeds: UserSeedDocument[] = [];

    consoleLog(`Creating ${noOffices} Office(s)`);
    await Promise.all(
      // exclude intiial office created w/company
      [...Array(noOffices - 1)].map(async (value, index) => {
        const getRandOfficeStart = getNow();
        const officeSeed = await db.officeseeds.getRandom();
        getDuration(getRandOfficeStart, 'Get Random Office Seed');
        consoleLog(
          `Creating office (${index + 1}/${noOffices}): ${officeSeed.name}`
        );
        const createOfficeStart = getNow();
        const office = await officeSeed.createOffice(company, sessionToken);
        offices.push(office);
        consoleLog(`Office ${office.get('name')} created!`, 'success');
        getDuration(createOfficeStart, 'Rand Office Created');
      })
    );

    consoleLog(`Creating ${noUsers} User(s)`);
    // Account for intial admin user & office created w/company
    const usersNo = noUsers - 1;
    const usersPerOffice = Math.floor(usersNo / noOffices);
    const remainder = usersNo % noOffices;
    const createUser = async (office: Office) => {
      const getUserSeed = getNow();
      const userSeed = await db.userseeds.getRandom();
      userSeeds.push(userSeed);
      getDuration(getUserSeed, 'Get User Seed');
      consoleLog(`Creating user: ${userSeed.userName}`);
      const createUserStart = getNow();
      const user = await userSeed.createUser(office, sessionToken);
      getDuration(createUserStart, 'Create Parse User');
      users.push(user);
      consoleLog(
        `User: ${userSeed.userName} added to office: ${office.get('name')}`,
        'success'
      );
    };
    if (usersNo > 0) {
      for (const office of offices) {
        for (let i = 0; i < usersPerOffice; i++) {
          await createUser(office);
        }
      }
      // Put all remainder users in initial company office
      if (remainder) {
        for (let i = 0; i < remainder; i++) {
          await createUser(offices[0]);
        }
      }
    }

    consoleLog(`Creating ${noMeasureItems} Measuresheet Item(s)`);
    const measureSheetItems: MeasureSheetItem[] = [];
    for (let i = 0; i < noMeasureItems; i++) {
      const randPriceSeedStart = getNow();
      const {
        category,
        itemName,
        itemNote,
        items,
        measurementType,
        orderNumber,
        shouldShowSwitch,
        subCategory,
        subSubCategory,
        subSubcategories,
      } = await db.priceguideseeds.getRandom(company);
      getDuration(randPriceSeedStart, 'Get Radom Measuresheet Item Seed');
      consoleLog(
        // eslint-disable-next-line max-len
        `Creating Measuresheet Item (${
          i + 1
        }/${noMeasureItems}) "${itemName}" with ${
          items.length
        } Priceguide Item(s)`
      );
      const createMSSstart = getNow();
      const measureSheetItem = await MeasureSheetItem.create(
        {
          company,
          category,
          includedOffices: offices,
          itemName,
          itemNote,
          items,
          measurementType,
          orderNumber,
          shouldShowSwitch,
          subCategory,
          subSubCategory,
          subSubcategories,
        },
        company,
        offices,
        sessionToken
      );
      getDuration(createMSSstart, 'Create Measuresheet Item');
      measureSheetItems.push(measureSheetItem);
      consoleLog(
        `Measuresheet Item "${measureSheetItem.get('itemName')}" created`,
        'success'
      );
    }

    const customers: Customer[] = [];
    for (let i = 0; i < noEstimates; i++) {
      consoleLog(`Creating estimate (${i + 1}/${noEstimates})`);
      const newCustomerStart = getNow();
      const randUserIndex = getRand(users.length - 1);
      const creator = await users[randUserIndex]?.fetch({
        useMasterKey: true,
      });
      const officeIds: string[] = creator?.get('allowedOffices');
      const officeId = officeIds[getRand(officeIds.length - 1)];
      const office = new Office();
      office.id = officeId;
      let userName: string;
      let passwd: string;
      if (randUserIndex === 0) {
        [userName, passwd] = [username, password];
      } else {
        const _username = creator?.getUsername();
        const userDoc = await db.userseeds
          .findOne()
          .where('userName')
          .equals(_username)
          .exec();
        if (userDoc) {
          ({ userName, password: passwd } = userDoc);
        } else {
          [userName, passwd] = ['', ''];
        }
      }
      if (userName !== '' && passwd !== '' && creator) {
        const token = await getSessionToken(userName, passwd);
        const contactEmailSeed = await db.contactseeds.getRandom(2);
        const contactEmail = await contactEmailSeed.createContact(
          company,
          token
        );
        const contactPhoneSeed = await db.contactseeds.getRandom(1);
        const contactPhone = await contactPhoneSeed.createContact(
          company,
          token
        );
        const addressSeed = await db.addressseeds.getRandom();
        const address = await addressSeed.createAddress(company, token);
        const estimateSeed = await db.estimateseeds.getRandom();
        const estimate = await estimateSeed.createEstimate(office, company);
        const customerSeed = await db.customerseeds.getRandom();
        getDuration(newCustomerStart, 'Build Customer from Random Seed');
        try {
          const createCustomerStart = getNow();
          const customer = await customerSeed.createCustomer(
            contactPhone,
            contactEmail,
            address,
            estimate,
            company,
            creator,
            token
          );
          getDuration(createCustomerStart, 'Create Customer');
          customers.push(customer);
        } catch (error) {
          consoleLog(error, 'danger');
          throw error;
        }
      }

      const partnerCreditAppQuery = new Parse.Query(PartnerCreditApp);
      const partnerCreditApps = await partnerCreditAppQuery.find({
        useMasterKey: true,
      });
      const creditApps: CreditApp[] = [];
      for (let i = 0; i < noCreditApps; i++) {
        const getRandCreditApp = getNow();
        const creditAppDoc = await db.creditappseeds.getRandom();
        const creator = users[getRand(users.length - 1)];
        const customer = customers[getRand(customers.length - 1)];
        const partnerApp =
          partnerCreditApps[getRand(partnerCreditApps.length - 1)];
        getDuration(getRandCreditApp, 'Get Random Credit App');
        const createCreditAppStart = getNow();
        if (creator) {
          const creditApp = await creditAppDoc.createCreditApp(
            company,
            creator,
            customer,
            sessionToken,
            partnerApp,
            creditLogoFile
          );
          getDuration(createCreditAppStart, 'Create Credit App');
          creditApps.push(creditApp);
        }
      }

      consoleLog(`Creating ${noContracts} contract(s)`);
      const contracts = [];
      for (let i = 0; i < noContracts; i++) {
        consoleLog(`Creating contract (${i + 1}/${noEstimates})`, 'success');
        const getRandContractStart = getNow();
        const randUserIndex = getRand(users.length - 1);
        const creator = await users[randUserIndex]?.fetch({
          useMasterKey: true,
        });
        if (creator) {
          let [userName, passwd] = ['', ''];
          if (randUserIndex === 0) {
            userName = username;
            passwd = password;
          } else {
            const _username = creator?.getUsername();
            const userDoc = await db.userseeds
              .findOne()
              .where('userName')
              .equals(_username)
              .exec();
            if (userDoc) {
              ({ userName, password: passwd } = userDoc);
            }
          }
          const token = await getSessionToken(userName, passwd);
          const _offices: Office[] = creator.get('allowedOffices');
          const {
            author,
            canAddMultiplePages,
            category,
            contractData,
            displayName,
            excludedStates,
            hMargin,
            iconBackgroundColor,
            isCorporateTemplate,
            isTemplate,
            order,
            pageId,
            pageSize,
            photosPerPage,
            type,
            updatedAt,
            useWatermark,
            watermarkAlpha,
            watermarkWidthPercent,
            wMargin,
          } = await db.contractseeds.getRandom();

          const mockContract: IMockContractObject = {
            author,
            canAddMultiplePages,
            category,
            // swap measuresheet item ids for ones on this company
            contractData: contractData.map((cdata) => ({
              ...cdata,
              data: cdata.data.map((data) => ({
                ...data,
                linkedMeasureSheetItemIds:
                  Array.isArray(data.linkedMeasureSheetItemIds) &&
                  data.linkedMeasureSheetItemIds.length > 0
                    ? data.linkedMeasureSheetItemIds.map(() => {
                        const randMS =
                          measureSheetItems[
                            getRand(measureSheetItems.length - 1)
                          ];
                        return randMS.id;
                      })
                    : [],
              })),
            })),
            company,
            displayName,
            excludedStates,
            file: testPdfFile,
            hMargin,
            iconBackgroundColor,
            iconImageThumb: testPdfThumbnailFile,
            images: [],
            includedOffices: _offices,
            isCorporateTemplate,
            isTemplate,
            lastModifiedBy: creator,
            order,
            pageId,
            pageSize,
            pdf: testPdfFile,
            pdfThumbnail: testPdfThumbnailFile,
            photosPerPage,
            preview: testPdfThumbnailFile,
            type,
            updatedAt,
            useWatermark,
            watermarkAlpha,
            watermarkWidthPercent,
            wMargin,
          };
          getDuration(getRandContractStart, 'Get Random Contract Seed');
          const createContractStart = getNow();
          const contract = await ContractObject.create(mockContract, token);
          getDuration(createContractStart, 'Create Contract');
          contracts.push(contract);
        }
      }
    }

    const created = moment();
    const log = `Company ${companyName} Created on ${created.toString()}

${noOffices} Offices
${noUsers} Users
${noMeasureItems} Measure Sheet Items
${noEstimates} Estimates
${noCreditApps} Credit Applications
${noContracts} Contracts

Company Administrator:
usr: ${username.padEnd(18, ' ')}pwd: ${password}

Users:
${userSeeds
  .map(
    ({ userName, password }) =>
      `usr: ${userName.padEnd(18, ' ')}pwd: ${password}\n`
  )
  .reduce((acc, val) => acc + val, '')}
`;
    const logName = `CreateCompany-${created.format('YYYY-MM-DD_HH.mm.ss')}`;
    const logPath = path.join(process.cwd(), logName);
    await new Promise((resolve, reject) => {
      fs.writeFile(logPath, log, { encoding: 'utf-8' }, (err) => {
        if (err) {
          reject(err);
        } else {
          resolve();
        }
      });
    });

    consoleLog('Company fully mocked!', 'success');
    consoleLog(`-Company Admin-`, 'success');
    consoleLog(`Username: ${username}; Password: ${password}`);
  } catch (e) {
    console.error(e);
    consoleLog(e, 'danger');
  }
};

export default generateRandCompany;
